//	BarsState.cpp - State for the 'Blade in my Chest' branch.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PhaseDriftSounder.h"
#include "drawing/Drawer.h"
#include "BarsState.h"
#include "Random.h"

//------------------------------------------------------------------------------
GameStateCreator<BarsState> BarsState::creator;

//------------------------------------------------------------------------------
BarsState::BarsState(Application& app, SounderPtr& sound):
GameState(app, sound),
tempBar(false),
rotVal(0.0f),
rotSpeed(0.001f),
rotSpeedDest(0.001f),
colourSpaceIndex(0.0f),
destColourSpaceIndex(0.0f),
sounder(dynamic_pointer_cast<PhaseDriftSounder>(sound))
{
	int i;
	int tempint;

	for(i=0;i<64;++i)
	{
		Bar tempBar;

		tempBar.pos = Random::getGlobal().getRandomFloat() * getDrawingAreaSize().x;
		tempBar.dir = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : -1.0f;
		tempBar.dir *= 0.25f + (Random::getGlobal().getRandomFloat() * 1.75f);

		//Colour space colours.
		//Black & white.
		/*tempint = Random::getGlobal().getRandomInt(2);
		switch(tempint)
		{
			case 0:
				tempBar.colourSpaceCols[0] = 0.0f;
				break;
			case 1:
				tempBar.colourSpaceCols[0] = 1.0f;
				break;
		}

		//Black, red & white.
		tempint = Random::getGlobal().getRandomInt(3);
		switch(tempint)
		{
			case 0:
				tempBar.colourSpaceCols[1] = 0.0f;
				break;
			case 1:
				tempBar.colourSpaceCols[1] = 1.0f;
				break;
			case 2:
				tempBar.colourSpaceCols[1] = 0.0f;
				tempBar.colourSpaceCols[1].x = 0.8f;
				break;
		}

		//Pastel blue, yellow, green, pink.
		tempint = Random::getGlobal().getRandomInt(4);
		switch(tempint)
		{
			case 0:
				tempBar.colourSpaceCols[2] = ThreeFloats(0.75f, 0.75f, 1.0f);
				break;
			case 1:
				tempBar.colourSpaceCols[2] = ThreeFloats(1.0f, 1.0f, 0.75f);
				break;
			case 2:
				tempBar.colourSpaceCols[2] = ThreeFloats((166.0f/255.0f),
														 (200.0f/255.0f),
														 (166.0f/255.0f));
				break;
			case 3:
				tempBar.colourSpaceCols[2] = ThreeFloats((254.0f/255.0f),
														 (164.0f/255.0f),
														 (164.0f/255.0f));
				break;
		}

		//Random.
		tempBar.colourSpaceCols[3].x = Random::getGlobal().getRandomFloat();
		tempBar.colourSpaceCols[3].y = Random::getGlobal().getRandomFloat();
		tempBar.colourSpaceCols[3].z = Random::getGlobal().getRandomFloat();*/

		updateBarColours(tempBar);
		tempBar.colourSpaceCols[0] = 1.0f;

		tempBar.col = tempBar.colourSpaceCols[0];
		
		tempint = Random::getGlobal().getRandomInt(4);
		switch(tempint)
		{
			case 0:
				tempBar.image = L"Bar0";
				tempBar.width = 32.0f;
				break;
			case 1:
				tempBar.image = L"Bar1";
				tempBar.width = 64.0f;
				break;
			case 2:
				tempBar.image = L"Bar2";
				tempBar.width = 128.0f;
				break;
			case 3:
				tempBar.image = L"Bar3";
				tempBar.width = 256.0f;
				break;
		}
		bars.push_back(tempBar);
	}

	droneIndex = sounder->activateSound(L"Music_BarsDrone", TwoFloats(0.5f));
	sounder->registerCallback(droneIndex, 2.5f, this);

	setMousePos(TwoFloats(0.0f, 360.0f));
}

//------------------------------------------------------------------------------
BarsState::~BarsState()
{
	sounder->unregisterCallback(droneIndex, this);
}

//------------------------------------------------------------------------------
void BarsState::update()
{
	int i;
	const float delta = 1.0f;
	const float screenWidth = getDrawingAreaSize().x;

	for(i=0;i<bars.size();++i)
	{
		bars[i].pos += bars[i].dir * delta;

		if(bars[i].pos > screenWidth)
		{
			bars[i].dir = -bars[i].dir;
			updateBarColours(bars[i]);
		}
		else if((bars[i].pos+bars[i].width) < 0.0f)
		{
			bars[i].dir = -bars[i].dir;
			updateBarColours(bars[i]);
		}
	}

	if(rotSpeed != rotSpeedDest)
	{
		rotSpeed += (rotSpeedDest-rotSpeed) * 0.01f * delta;
		if(fabsf(rotSpeedDest-rotSpeed) < 0.000001f)
			rotSpeed = rotSpeedDest;
	}

	rotVal += rotSpeed * delta;

	if(colourSpaceIndex != destColourSpaceIndex)
	{
		colourSpaceIndex += (destColourSpaceIndex - colourSpaceIndex) * 0.05f;
		if(fabsf(destColourSpaceIndex-colourSpaceIndex) < 0.05f)
			colourSpaceIndex = destColourSpaceIndex;

		for(i=0;i<bars.size();++i)
			calcInterpColour(bars[i]);
	}
}

//------------------------------------------------------------------------------
void BarsState::draw(DrawerPtr& d, float timestep)
{
	int i;
	const TwoFloats origin;
	const TwoFloats centre(d->getDrawingAreaSize() * 0.5f);

	d->fillRect(origin, d->getDrawingAreaSize(), ThreeFloats(1.0f));

	if(tempBar)
	{
		/*d->drawImage(L"BarSide",
					 TwoFloats(640.0f-32.0f-4.0f, 0.0f),
					 TwoFloats(1.0f, d->getDrawingAreaSize().y),
					 ThreeFloats(0.0f));
		d->drawImage(L"BarCentre",
					 TwoFloats(640.0f-32.0f, 0.0f),
					 TwoFloats(16.0f, d->getDrawingAreaSize().y/4.0f),
					 ThreeFloats(0.0f));
		d->drawImage(L"BarSide",
					 TwoFloats(640.0f+32.0f+4.0f, 0.0f),
					 TwoFloats(-1.0f, d->getDrawingAreaSize().y),
					 ThreeFloats(0.0f));
		d->drawImage(L"Bar0",
					 TwoFloats(640.0f-16.0f, 0.0f),
					 TwoFloats(1.0f, d->getDrawingAreaSize().y),
					 ThreeFloats(0.0f));*/
	}

	d->rotate(rotVal, centre);

	for(i=0;i<bars.size();++i)
	{
		d->drawImage(bars[i].image,
					 TwoFloats(bars[i].pos, -(centre.x-centre.y)-(2.5f*(d->getDrawingAreaSize().x/24.0f))),
					 TwoFloats(1.0f, (d->getDrawingAreaSize().x/24.0f)),
					 bars[i].col);
	}
}

//------------------------------------------------------------------------------
void BarsState::keyDown(const std::wstring& action,
						const std::wstring& keyPressed,
						int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void BarsState::mouseMove(const TwoFloats& pos)
{
	rotSpeedDest = (((pos.y/getDrawingAreaSize().y)*2.0f)-1.0f) * 0.002f;

	destColourSpaceIndex = (pos.x/getDrawingAreaSize().x) *
						   ((float)NumColourSpaces - 1.01f);
}

//------------------------------------------------------------------------------
void BarsState::reachedTime(const std::wstring& name, float time)
{
	if((name == L"Music_BarsDrone") && (time == 2.5f))
		tempBar = true;
}

//------------------------------------------------------------------------------
void BarsState::updateBarColours(Bar& bar)
{
	int tempint;

	//Colour space colours.
	//Black & white.
	tempint = Random::getGlobal().getRandomInt(2);
	switch(tempint)
	{
		case 0:
			/*bar.colourSpaceCols[0] = 0.0f;
			break;*/
		case 1:
			bar.colourSpaceCols[0] = 1.0f;
			break;
	}

	//Black, red & white.
	tempint = Random::getGlobal().getRandomInt(3);
	switch(tempint)
	{
		case 0:
			bar.colourSpaceCols[1] = 0.0f;
			break;
		case 1:
			bar.colourSpaceCols[1] = 1.0f;
			break;
		case 2:
			bar.colourSpaceCols[1] = 0.0f;
			bar.colourSpaceCols[1].x = 0.8f;
			break;
		/*case 0:
			bar.colourSpaceCols[1] = 0.5f;
			break;
		case 1:
			bar.colourSpaceCols[1] = ThreeFloats(1.0f, 0.5f, 0.5f);
			break;
		case 2:
			bar.colourSpaceCols[1] = ThreeFloats(0.5f, 0.5f, 1.0f);
			break;*/
	}

	//Pastel blue, yellow, green, pink.
	tempint = Random::getGlobal().getRandomInt(4);
	switch(tempint)
	{
		case 0:
			bar.colourSpaceCols[2] = ThreeFloats(0.75f, 0.75f, 1.0f);
			break;
		case 1:
			bar.colourSpaceCols[2] = ThreeFloats(1.0f, 1.0f, 0.75f);
			break;
		case 2:
			bar.colourSpaceCols[2] = ThreeFloats((166.0f/255.0f),
												 (200.0f/255.0f),
												 (166.0f/255.0f));
			break;
		case 3:
			bar.colourSpaceCols[2] = ThreeFloats((254.0f/255.0f),
														(164.0f/255.0f),
														(164.0f/255.0f));
			break;
	}

	//Random.
	bar.colourSpaceCols[0].x = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[0].y = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[0].z = Random::getGlobal().getRandomFloat();

	bar.colourSpaceCols[1].x = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[1].y = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[1].z = Random::getGlobal().getRandomFloat();

	bar.colourSpaceCols[2].x = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[2].y = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[2].z = Random::getGlobal().getRandomFloat();

	bar.colourSpaceCols[3].x = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[3].y = Random::getGlobal().getRandomFloat();
	bar.colourSpaceCols[3].z = Random::getGlobal().getRandomFloat();
}

//------------------------------------------------------------------------------
void BarsState::calcInterpColour(Bar& bar)
{
	int index = (int)colourSpaceIndex;
	float fraction = colourSpaceIndex - (float)index;

	bar.col = bar.colourSpaceCols[index] * (1.0f-fraction);
	bar.col += bar.colourSpaceCols[index+1] * fraction;
}
