//	BladeState.cpp - State for the 'Blade in my Chest' branch.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "BladeState.h"
#include "Random.h"

using namespace std;

//------------------------------------------------------------------------------
GameStateCreator<BladeState> BladeState::creator;

//------------------------------------------------------------------------------
BladeState::BladeState(Application& app, SounderPtr& sound):
GameState(app, sound),
inc(0.0f),
circleCount(0),
lastMouse(640.0f, 360.0f)
{
	circles.push_back(Circle());

	circles[0].radius = 0.05f;
	circles[0].colour = ThreeFloats(1.0f, (25.0f/255.0f), (26.0f/255.0f));

	setMousePos(lastMouse);
}

//------------------------------------------------------------------------------
BladeState::~BladeState()
{

}

//------------------------------------------------------------------------------
void BladeState::update()
{
	int i;

	if(inc < 0.002f)
		inc += 0.00001f;
	if(circleCount)
		--circleCount;

	for(i=((int)circles.size()-1);i>=0;--i)
	{
		circles[i].radius += inc;
		if(circles[i].radius >= 1.0f)
		{
			backColour = circles[i].colour;
			circles.erase(circles.begin()+i);
		}
	}
}

//------------------------------------------------------------------------------
void BladeState::draw(DrawerPtr& d, float timestep)
{
	unsigned int i;
	const TwoFloats origin;
	const TwoFloats centre = d->getDrawingAreaSize() * 0.5f;

	d->fillRect(origin, d->getDrawingAreaSize(), backColour);

	for(i=0;i<circles.size();++i)
	{
		d->drawImage(L"BigCircle",
					 centre-(circles[i].radius * 1024.0f),
					 circles[i].radius,
					 circles[i].colour);
	}
}

//------------------------------------------------------------------------------
void BladeState::keyDown(const wstring& action,
						 const wstring& keyPressed,
						 int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void BladeState::mouseMove(const TwoFloats& pos)
{
	float dist;
	float randG, randB;

	if((inc >= 0.002f) && !circleCount)
	{
		dist = pos.getDistance(lastMouse);
		if(dist > 8.0f)
		{
			circles.push_back(Circle());
			circles.back().radius = 0.0f;
			dist -= 8.0f;
			dist /= 64.0f;
			if(dist > 1.0f)
				dist = 1.0f;
			randG = Random::getGlobal().getRandomFloat() * 0.08f;
			randB = Random::getGlobal().getRandomFloat() * 0.1f;
			circles.back().colour = ThreeFloats(dist*0.8f,
												randG*dist,
												randB*dist);
		}
		circleCount = 30;
	}
	lastMouse = pos;
}
