//	BladeState.h - State for the 'Blade in my Chest' branch.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BLADESTATE_H_
#define BLADESTATE_H_

#include "ThreeFloats.h"
#include "GameState.h"

#include <vector>

///	State for the 'Blade in my Chest' branch.
class BladeState : public GameState
{
  public:
	///	Constructor.
	BladeState(Application& app, SounderPtr& sound);
	///Destructor.
	~BladeState();

	///	Called frequently to update the state.
	void update();
	///	Called to draw the state.
	void draw(DrawerPtr& d, float timestep);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when a mouse move event is received.
	void mouseMove(const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"BladeState";};
	///	Used to construct instances of this state.
	static GameStateCreator<BladeState> creator;
  private:
	///	Struct representing a circle.
	struct Circle
	{
		///	The circle's radius.
		float radius;
		///	The circle's colour.
		ThreeFloats colour;
	};

	///	The circles currently visible.
	std::vector<Circle> circles;
	///	The current background colour.
	ThreeFloats backColour;

	///	Used to start the 1st circle moving slowly.
	float inc;
	///	Used to avoid triggering new circles too soon.
	int circleCount;

	///	The previous mouse position.
	TwoFloats lastMouse;
};

#endif
