//	OctopusState.h - The state with 'octopus' arms.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef OCTOPUSSTATE_H_
#define OCTOPUSSTATE_H_

#include "GameState.h"

#include <vector>

///	The state with 'octopus' arms.
class OctopusState : public GameState
{
  public:
	///	Constructor.
	OctopusState(Application& app, SounderPtr& sound);
	///	Destructor.
	~OctopusState();
	
	///	Updates the state's motion.
	void update();
	///	Draws the state.
	void draw(DrawerPtr& d, float timestep);

	///	Used to quit on escape.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when a mouse move event is received.
	void mouseMove(const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"OctopusState";};
	///	Used to construct instances of this state.
	static GameStateCreator<OctopusState> creator;
  private:
	///	Some constants.
	enum
	{
		NumArms = 16
	};

	///	Represents a single arm.
	struct Arm
	{
		///	The arm's angle around the centre point.
		float angle;
		///	The arm's speed.
		float speed;
		///	The two sine's speeds.
		float sinSpeed[2];
		///	The two sine's amplitudes.
		float sinAmp[2];
		///	The two sine's offsets.
		float sinOffset[2];
	};
	///	Represents a dot on an Arm.
	struct Dot
	{
		///	Index of the Arm this Dot is on.
		int armIndex;
		///	The dot's position on its arm.
		/*!
			If this is greater than 1, the Dot has left the Arm.
		 */
		float armPos;
		///	The Dot's position on the screen.
		TwoFloats position;
		///	The Dot's current direction.
		TwoFloats direction;
		///	The Dot's current speed.
		float speed;
		///	The Dot's alpha value.
		float alpha;

		///	Constructor.
		Dot():
		armPos(0.0f),
		alpha(0.0f)
		{

		};
	};

	///	The octopus's arms.
	Arm arms[NumArms];
	///	The dots.
	std::vector<Dot> dots;

	///	Timer used to determine when to add new Dots.
	int newDotTimer;

	///	The current mouse position.
	TwoFloats mousePos;
	///	The current arms centre position.
	TwoFloats centrePos;
};

#endif
