//	PhaseDriftState.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PHASEDRIFTSTATE_H_
#define PHASEDRIFTSTATE_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "GameState.h"

class PhaseDriftSounder;

///	The main gameplay state.
class PhaseDriftState : public GameState
{
  public:
	///	Constructor.
	PhaseDriftState(Application& app, SounderPtr& sound);
	///	Destructor.
	~PhaseDriftState();

	///	Draws the current state.
	void draw(DrawerPtr& d, float timestep);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when a mouse move event is received.
	void mouseMove(const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"PhaseDriftState";};
	///	Used to construct instances of this state.
	static GameStateCreator<PhaseDriftState> creator;
  private:
	///	Rotates the points.
	float rot;
	///	The speed of the rotation.
	float rotSpeed;

	///	Mouse position (0->1).
	TwoFloats mousePos;
	///	Drawing area size.
	TwoFloats drawSize;

	///	Used to fade in at the start.
	float fadeIn;

	///	The positions of the various sound sources.
	std::vector<TwoFloats> sourcesPos;
	///	The indices of the various sound sources.
	std::vector<int> sourcesIndices;
	///	Index of the sound source the mousePos is being moved towards.
	/*!
		Will be -1 if it's too far away from any sources.
	 */
	int currentSource;
	///	Used to determine when to start moving the mouse towards a sound source.
	int staticCount;
	///	Whether the mouse has already been moved to a sound source.
	bool foundSource;

	///	The colour of the background.
	ThreeFloats backColour;
	///	Used to fade to blue if the player follows that branch.
	float backFade;
	///	Used to gradually reduce the number of points on the blue branch.
	float bounds;
	///	Used to gradually reduce the radius on the 'Blade...' branch.
	float bladeFade;

	///	Used to determine when the player has followed the 'Our Stories are Wrong' branch.
	int storiesCount;
	///	Used to determine when the player has followed the 'Blade in my Chest' branch.
	int bladeCount;
	///	Used to determine when to switch to the PhaseDrift2 state.
	int phaseDrift2Count;

	///	Our sounder.
	shared_ptr<PhaseDriftSounder> sounder;
};

#endif
