//	StoriesState.h - The 'Our Stories are Wrong' state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef STORIESSTATE_H_
#define STORIESSTATE_H_

#include "GameState.h"

///	The 'Our Stories are Wrong' state.
class StoriesState : public GameState
{
  public:
	///	Constructor.
	StoriesState(Application& app, SounderPtr& sound);
	///	Destructor.
	~StoriesState();

	///	Updates the state's motion.
	void update();
	///	Draws the state.
	void draw(DrawerPtr& d, float timestep);

	///	Used to quit on escape.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when a mouse move event is received.
	void mouseMove(const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"StoriesState";};
	///	Used to construct instances of this state.
	static GameStateCreator<StoriesState> creator;
  private:
	///	The number of vectors we have.
	enum
	{
		NumVectors = 32,
		NumPoints = 14
	};

	///	Drawing area size.
	TwoFloats drawSize;

	///	The start points of the point vectors.
	TwoFloats pointsPos[NumVectors];
	///	The directions of the point vectors.
	TwoFloats pointsDir[NumVectors];

	///	How far along the vector the points are.
	float pointsIndex;
	///	Used to lengthen the vectors when the pointsPos radius changes.
	float radius;

	///	The current mouse position.
	TwoFloats mousePos;
	///	The desired mouse position.
	TwoFloats desiredPos;

	///	Used to make sure the points are centred correctly.
	/*!
		Because the aspect ratio could be anything, we need to centre the points
		in the draw() call.
	 */
	bool justStarted;
	///	Used to make sure the points are centred correctly.
	TwoFloats centre;
};

#endif
